@@ -23,12 +23,13 @@ from rasa.cli import (
     train,
     visualize,
     x,
+    evaluate,
 )
 from rasa.cli.arguments.default_arguments import add_logging_options
 from rasa.cli.utils import parse_last_positional_argument_as_model_path
 from rasa.shared.exceptions import RasaException
 from rasa.shared.utils.cli import print_error
-from rasa.utils.common import set_log_and_warnings_filters, set_log_level
+from rasa.utils.common import configure_logging_and_warnings
 
 logger = logging.getLogger(__name__)
 
@@ -69,24 +70,16 @@ def create_argument_parser() -> argparse.ArgumentParser:
     data.add_subparser(subparsers, parents=parent_parsers)
     export.add_subparser(subparsers, parents=parent_parsers)
     x.add_subparser(subparsers, parents=parent_parsers)
+    evaluate.add_subparser(subparsers, parents=parent_parsers)
 
     return parser
 
 
 def print_version() -> None:
     """Prints version information of rasa tooling and python."""
-
-    try:
-        from rasax.community.version import __version__
-
-        rasa_x_info = __version__
-    except ModuleNotFoundError:
-        rasa_x_info = None
-
     print(f"Rasa Version      :         {version.__version__}")
     print(f"Minimum Compatible Version: {MINIMUM_COMPATIBLE_VERSION}")
     print(f"Rasa SDK Version  :         {rasa_sdk_version}")
-    print(f"Rasa X Version    :         {rasa_x_info}")
     print(f"Python Version    :         {platform.python_version()}")
     print(f"Operating System  :         {platform.platform()}")
     print(f"Python Path       :         {sys.executable}")
@@ -98,10 +91,10 @@ def main() -> None:
     arg_parser = create_argument_parser()
     cmdline_arguments = arg_parser.parse_args()
 
-    log_level = (
-        cmdline_arguments.loglevel if hasattr(cmdline_arguments, "loglevel") else None
+    log_level = getattr(cmdline_arguments, "loglevel", None)
+    configure_logging_and_warnings(
+        log_level, warn_only_once=True, filter_repeated_logs=True
     )
-    set_log_level(log_level)
 
     tf_env.setup_tf_environment()
     tf_env.check_deterministic_ops()
@@ -112,7 +105,6 @@ def main() -> None:
     try:
         if hasattr(cmdline_arguments, "func"):
             rasa.utils.io.configure_colored_logging(log_level)
-            set_log_and_warnings_filters()
             rasa.telemetry.initialize_telemetry()
             rasa.telemetry.initialize_error_reporting()
             cmdline_arguments.func(cmdline_arguments)
