@@ -3,27 +3,49 @@ import logging
 import os
 import sys
 from types import FrameType
-from typing import Any, Dict, List, Optional, TYPE_CHECKING, Text
+from typing import Any, Dict, List, Optional, TYPE_CHECKING, Text, Union, overload
 
-from rasa.shared.constants import DEFAULT_MODELS_PATH
 import rasa.shared.utils.cli
 import rasa.shared.utils.io
 
 if TYPE_CHECKING:
+    from pathlib import Path
+
     from questionary import Question
+    from typing_extensions import Literal
 
 logger = logging.getLogger(__name__)
 
 FREE_TEXT_INPUT_PROMPT = "Type out your own message..."
 
 
+@overload
 def get_validated_path(
-    current: Optional[Text],
+    current: Optional[Union["Path", Text]],
     parameter: Text,
-    default: Optional[Text] = None,
+    default: Optional[Union["Path", Text]] = ...,
+    none_is_valid: "Literal[False]" = ...,
+) -> Union["Path", Text]:
+    ...
+
+
+@overload
+def get_validated_path(
+    current: Optional[Union["Path", Text]],
+    parameter: Text,
+    default: Optional[Union["Path", Text]] = ...,
+    none_is_valid: "Literal[True]" = ...,
+) -> Optional[Union["Path", Text]]:
+    ...
+
+
+def get_validated_path(
+    current: Optional[Union["Path", Text]],
+    parameter: Text,
+    default: Optional[Union["Path", Text]] = None,
     none_is_valid: bool = False,
-) -> Optional[Text]:
-    """Check whether a file path or its default value is valid and returns it.
+) -> Optional[Union["Path", Text]]:
+    """Checks whether a file path or its default value is valid and returns it.
 
     Args:
         current: The parsed value.
@@ -57,7 +79,18 @@ def get_validated_path(
     return current
 
 
-def missing_config_keys(path: Text, mandatory_keys: List[Text]) -> List[Text]:
+def missing_config_keys(
+    path: Union["Path", Text], mandatory_keys: List[Text]
+) -> List[Text]:
+    """Checks whether the config file at `path` contains the `mandatory_keys`.
+
+    Args:
+        path: The path to the config file.
+        mandatory_keys: A list of mandatory config keys.
+
+    Returns:
+        The list of missing config keys.
+    """
     import rasa.utils.io
 
     if not os.path.exists(path):
@@ -69,7 +102,9 @@ def missing_config_keys(path: Text, mandatory_keys: List[Text]) -> List[Text]:
 
 
 def cancel_cause_not_found(
-    current: Optional[Text], parameter: Text, default: Optional[Text]
+    current: Optional[Union["Path", Text]],
+    parameter: Text,
+    default: Optional[Union["Path", Text]],
 ) -> None:
     """Exits with an error because the given path was not valid.
 
@@ -79,7 +114,6 @@ def cancel_cause_not_found(
         default: The default value of the parameter.
 
     """
-
     default_clause = ""
     if default:
         default_clause = f"use the default location ('{default}') or "
@@ -92,7 +126,6 @@ def cancel_cause_not_found(
 
 def parse_last_positional_argument_as_model_path() -> None:
     """Fixes the parsing of a potential positional model path argument."""
-
     if (
         len(sys.argv) >= 2
         # support relevant commands ...
@@ -106,39 +139,8 @@ def parse_last_positional_argument_as_model_path() -> None:
         sys.argv[-2] = "--model"
 
 
-def create_output_path(
-    output_path: Text = DEFAULT_MODELS_PATH,
-    prefix: Text = "",
-    fixed_name: Optional[Text] = None,
-) -> Text:
-    """Creates an output path which includes the current timestamp.
-
-    Args:
-        output_path: The path where the model should be stored.
-        fixed_name: Name of the model.
-        prefix: A prefix which should be included in the output path.
-
-    Returns:
-        The generated output path, e.g. "20191201-103002.tar.gz".
-    """
-    import time
-
-    if output_path.endswith("tar.gz"):
-        return output_path
-    else:
-        if fixed_name:
-            name = fixed_name
-        else:
-            time_format = "%Y%m%d-%H%M%S"
-            name = time.strftime(time_format)
-            name = f"{prefix}{name}"
-        file_name = f"{name}.tar.gz"
-        return os.path.join(output_path, file_name)
-
-
 def button_to_string(button: Dict[Text, Any], idx: int = 0) -> Text:
     """Create a string representation of a button."""
-
     title = button.pop("title", "")
 
     if "payload" in button:
