@@ -11,7 +11,7 @@ import numpy as np
 import scipy.sparse
 import tensorflow as tf
 
-from typing import Any, Dict, List, Optional, Text, Tuple, Union, Type
+from typing import Any, Dict, List, Optional, Text, Tuple, Union, TypeVar, Type
 
 from rasa.engine.graph import ExecutionContext, GraphComponent
 from rasa.engine.recipes.default_recipe import DefaultV1Recipe
@@ -117,6 +117,9 @@ LABEL_SUB_KEY = IDS
 POSSIBLE_TAGS = [ENTITY_ATTRIBUTE_TYPE, ENTITY_ATTRIBUTE_ROLE, ENTITY_ATTRIBUTE_GROUP]
 
 
+DIETClassifierT = TypeVar("DIETClassifierT", bound="DIETClassifier")
+
+
 @DefaultV1Recipe.register(
     [
         DefaultV1Recipe.ComponentType.INTENT_CLASSIFIER,
@@ -671,7 +674,7 @@ class DIETClassifier(GraphComponent, IntentClassifier, EntityExtractorMixin):
         if self._label_data is None:
             return []
 
-        feature_arrays: List[FeatureArray] = self._label_data.get(LABEL, SENTENCE)
+        feature_arrays = self._label_data.get(LABEL, SENTENCE)
         all_label_features = feature_arrays[0]
         return [
             FeatureArray(
@@ -1085,13 +1088,13 @@ class DIETClassifier(GraphComponent, IntentClassifier, EntityExtractorMixin):
 
     @classmethod
     def load(
-        cls,
+        cls: Type[DIETClassifierT],
         config: Dict[Text, Any],
         model_storage: ModelStorage,
         resource: Resource,
         execution_context: ExecutionContext,
         **kwargs: Any,
-    ) -> DIETClassifier:
+    ) -> DIETClassifierT:
         """Loads a policy from the storage (see parent class for full docstring)."""
         try:
             with model_storage.read_from(resource) as model_path:
@@ -1107,13 +1110,13 @@ class DIETClassifier(GraphComponent, IntentClassifier, EntityExtractorMixin):
 
     @classmethod
     def _load(
-        cls,
+        cls: Type[DIETClassifierT],
         model_path: Path,
         config: Dict[Text, Any],
         model_storage: ModelStorage,
         resource: Resource,
         execution_context: ExecutionContext,
-    ) -> "DIETClassifier":
+    ) -> DIETClassifierT:
         """Loads the trained model from the provided directory."""
         (
             index_label_id_mapping,
@@ -1566,7 +1569,7 @@ class DIET(TransformerRasaModel):
         )
 
     def batch_loss(
-        self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
+        self, batch_in: Union[Tuple[tf.Tensor, ...], Tuple[np.ndarray, ...]]
     ) -> tf.Tensor:
         """Calculates the loss for the given batch.
 
@@ -1730,7 +1733,7 @@ class DIET(TransformerRasaModel):
             _, self.all_labels_embed = self._create_all_labels()
 
     def batch_predict(
-        self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
+        self, batch_in: Union[Tuple[tf.Tensor, ...], Tuple[np.ndarray, ...]]
     ) -> Dict[Text, tf.Tensor]:
         """Predicts the output of the given batch.
 
